@@ -236,3 +236,9 @@ bool InfiniteAreaLight::bounding_box(Float t0, Float t1, aabb& box) const {
   box = (*ObjectToWorld)(aabb(-vec3f(radius,radius,radius), vec3f(radius,radius,radius)));
   return(true);
 }
+
+size_t InfiniteAreaLight::GetSize()  {
+  return(mat_ptr ? 
+           sizeof(*this) + distribution->GetSize() + sizeof(Float) * width * height + mat_ptr->GetSize() :
+           sizeof(*this) + distribution->GetSize() + sizeof(Float) * width * height);
+}
