@@ -2,9 +2,14 @@
 #include "RcppThread.h"
 
 bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng) {
-  point3f p0t = a - vec3f(r.origin());
-  point3f p1t = b - vec3f(r.origin());
-  point3f p2t = c - vec3f(r.origin());
+  const point3f &p0 = mesh->p[v[0]];
+  const point3f &p1 = mesh->p[v[1]];
+  const point3f &p2 = mesh->p[v[2]];
+  
+  point3f p0t = p0 - vec3f(r.origin());
+  point3f p1t = p1 - vec3f(r.origin());
+  point3f p2t = p2 - vec3f(r.origin());
+  
   int kz = MaxDimension(Abs(r.direction()));
   int kx = kz + 1;
   if (kx == 3) kx = 0;
@@ -14,7 +19,7 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
   p0t = Permute(p0t, kx, ky, kz);
   p1t = Permute(p1t, kx, ky, kz);
   p2t = Permute(p2t, kx, ky, kz);
-  
+
   // Apply shear transformation to translated vertex positions
   Float Sx = -d.x() / d.z();
   Float Sy = -d.y() / d.z();
@@ -29,7 +34,7 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
   Float e0 = DifferenceOfProducts(p1t.x(), p2t.y(), p1t.y(), p2t.x());
   Float e1 = DifferenceOfProducts(p2t.x(), p0t.y(), p2t.y(), p0t.x());
   Float e2 = DifferenceOfProducts(p0t.x(), p1t.y(), p0t.y(), p1t.x());
-  
+
   // Fall back to double precision test at triangle edges
   if (sizeof(Float) == sizeof(float) &&
       (e0 == 0.0f || e1 == 0.0f || e2 == 0.0f)) {
@@ -43,12 +48,12 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
     double p1typ0tx = (double)p1t.y() * (double)p0t.x();
     e2 = (float)(p1typ0tx - p1txp0ty);
   }
-  
+
   if ((e0 < 0 || e1 < 0 || e2 < 0) && (e0 > 0 || e1 > 0 || e2 > 0))
     return false;
   Float det = e0 + e1 + e2;
   if (det == 0) return false;
-  
+
   p0t[2] *= Sz;
   p1t[2] *= Sz;
   p2t[2] *= Sz;
@@ -58,7 +63,7 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
   } else if (det > 0 && (tScaled <= 0 || tScaled > t_max * det)) {
     return false;
   }
-  
+
   // Compute barycentric coordinates and $t$ value for triangle intersection
   Float invDet = 1 / det;
   Float b0 = e0 * invDet;
@@ -67,38 +72,30 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
   Float t = tScaled * invDet;
   Float maxZt = MaxComponent(Abs(vec3f(p0t.z(), p1t.z(), p2t.z())));
   Float deltaZ = gamma(3) * maxZt;
-  
+
   // Compute $\delta_x$ and $\delta_y$ terms for triangle $t$ error bounds
   Float maxXt = MaxComponent(Abs(vec3f(p0t.x(), p1t.x(), p2t.x())));
   Float maxYt = MaxComponent(Abs(vec3f(p0t.y(), p1t.y(), p2t.y())));
   Float deltaX = gamma(5) * (maxXt + maxZt);
   Float deltaY = gamma(5) * (maxYt + maxZt);
-  
+
   // Compute $\delta_e$ term for triangle $t$ error bounds
   Float deltaE = 2 * (gamma(2) * maxXt * maxYt + deltaY * maxXt + deltaX * maxYt);
-  
+
   // Compute $\delta_t$ term for triangle $t$ error bounds and check _t_
   Float maxE = MaxComponent(Abs(vec3f(e0, e1, e2)));
   Float deltaT = 3 *
     (gamma(3) * maxE * maxZt + deltaE * maxZt + deltaZ * maxE) *
     std::abs(invDet);
   if (t <= deltaT) return false;
-  
+
   vec3f dpdu, dpdv;
   point2f uv[3];
-  if(uv_provided) {
-    uv[0] = uv_a;
-    uv[1] = uv_b;
-    uv[2] = uv_c;
-  } else {
-    uv[0] = point2f(0, 0);
-    uv[1] = point2f(1, 0);
-    uv[2] = point2f(1, 1);
-  }
-  
+  GetUVs(uv);
+
   // Compute deltas for triangle partial derivatives
   vec2f duv02 = uv[0] - uv[2], duv12 = uv[1] - uv[2];
-  vec3f dp02 = a - c, dp12 = b - c;
+  vec3f dp02 = p0 - p2, dp12 = p1 - p2;
   Float determinant = DifferenceOfProducts(duv02[0],duv12[1],duv02[1],duv12[0]);
   bool degenerateUV = std::fabs(determinant) < 1e-8;
   if (!degenerateUV) {
@@ -108,7 +105,7 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
   }
   if (degenerateUV || cross(rec.dpdu, rec.dpdv).squared_length() == 0) {
     // Handle zero determinant for triangle partial derivative matrix
-    vec3f ng = cross(c - a, b - a);
+    vec3f ng = cross(p2 - p0, p1 - p0);
     if (ng.squared_length() == 0) {
       // The triangle is actually degenerate; the intersection is
       // bogus.
@@ -117,24 +114,31 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
     CoordinateSystem(unit_vector(ng), &rec.dpdu, &rec.dpdv);
   }
   //Add error calc
-  Float xAbsSum = (std::abs(b0 * a.x()) + std::abs(b1 * b.x()) +
-    std::abs(b2 * c.x()));
-  Float yAbsSum = (std::abs(b0 * a.y()) + std::abs(b1 * b.y()) +
-    std::abs(b2 * c.y()));
-  Float zAbsSum = (std::abs(b0 * a.z()) + std::abs(b1 * b.z()) +
-    std::abs(b2 * c.z()));
+  Float xAbsSum = (std::abs(b0 * p0.x()) + std::abs(b1 * p1.x()) +
+    std::abs(b2 * p2.x()));
+  Float yAbsSum = (std::abs(b0 * p0.y()) + std::abs(b1 * p1.y()) +
+    std::abs(b2 * p2.y()));
+  Float zAbsSum = (std::abs(b0 * p0.z()) + std::abs(b1 * p1.z()) +
+    std::abs(b2 * p2.z()));
   rec.pError = gamma(7) * vec3f(xAbsSum, yAbsSum, zAbsSum);
-  
-  point3f pHit = b0 * a + b1 * b + b2 * c;
-  point2f uvHit = b0 * uv[0] + b1 * uv[1] + b2 * uv[2];
 
-  Float u = uvHit[0];
-  Float v = uvHit[1];
+  point3f pHit = b0 * p0 + b1 * p1 + b2 * p2;
+  point2f uvHit = b0 * uv[0] + b1 * uv[1] + b2 * uv[2];
+  
+  Float uHit = uvHit[0];
+  Float vHit = uvHit[1];
+  if(mesh->has_vertex_colors) {
+    uHit = b0;
+    vHit = b1;
+  } 
   
   bool alpha_miss = false;
+  normal3f normal = normal3f(unit_vector(cross(dp02, dp12)));
+  int mat_id = mesh->face_material_id[face_number];
 
+  alpha_texture* alpha_mask = mesh->alpha_textures[mat_id].get();
   if(alpha_mask) {
-    if(alpha_mask->value(u, v, rec.p) < rng.unif_rand()) {
+    if(alpha_mask->value(uHit, vHit, rec.p) < rng.unif_rand()) {
       alpha_miss = true;
     }
   }
@@ -143,15 +147,31 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
   rec.pError = gamma(7) * vec3f(xAbsSum, yAbsSum, zAbsSum);
 
   rec.has_bump = false;
-  
+
   // Use that to calculate normals
-  if(normals_provided) {
-    point2f uvHit2 = b0 *  point2f(0, 0) + b1 * point2f(1, 0) + b2 * point2f(1, 1);
-    Float u1 = uvHit2[0];
-    Float v1 = uvHit2[1];
-    Float w = 1 - u1 - v1;
-    
-    rec.normal = unit_vector(w * na + u1 * nb + v1 * nc);
+  if(mesh->has_normals && n[0] != -1 && n[1] != -1 && n[2] != -1) {
+    normal3f n1 = mesh->n[n[0]];
+    normal3f n2 = mesh->n[n[1]];
+    normal3f n3 = mesh->n[n[2]];
+    normal3f np = unit_vector(b0 * n1 + b1 * n2 + b2 * n3);
+
+    if(mesh->has_consistent_normals) {
+      bool flip = dot(np, r.direction()) > 0;
+      Float af1 = mesh->alpha_v[n[0]];
+      Float af2 = mesh->alpha_v[n[1]];
+      Float af3 = mesh->alpha_v[n[2]];
+      Float af = b0 * af1 + b1 * af2 + b2 * af3;
+      vec3f i = -unit_vector(r.direction());
+      i *= flip ? -1 : 1;
+      Float b = dot(i,np);
+      Float q = (1 - 2 * M_1_PI * af) * (1 - (2 * M_1_PI) * af)/(1 + 2 * (1 - 2 * M_1_PI) * af);
+      Float g = 1 + q * (b - 1);
+      Float rho = sqrt(q * (1 + g) / (1 +b));
+      normal3f r1 = (g + rho * b) * np - rho * normal3f(i.x(),i.y(),i.z());
+      rec.normal = unit_vector(normal3f(i.x(),i.y(),i.z()) + r1);
+    } else {
+      rec.normal = np;
+    }
   } else {
     if(alpha_mask) {
       rec.normal = dot(r.direction(), normal) < 0 ? normal : -normal;
@@ -159,19 +179,22 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
       rec.normal = normal;
     }
   }
-
+  bump_texture* bump_tex = mesh->bump_textures[mat_id].get();
+  
   if(bump_tex) {
-    vec3f norm_bump = dot(r.direction(), normal) < 0 ? rec.normal.convert_to_vec3() : -rec.normal.convert_to_vec3();
-    
-    point3f bvbu = bump_tex->value(u, v, rec.p);
+    vec3f norm_bump = dot(r.direction(), rec.normal) < 0 ? rec.normal.convert_to_vec3() : -rec.normal.convert_to_vec3();
+
+    point3f bvbu = bump_tex->value(uHit, vHit, rec.p);
     rec.bump_normal = cross(rec.dpdu + bvbu.x() * norm_bump ,
                             rec.dpdv - bvbu.y() * norm_bump);
     rec.bump_normal.make_unit_vector();
+    rec.bump_normal = Faceforward(rec.bump_normal,rec.normal);
     rec.has_bump = true;
   }
-  rec.u = u;
-  rec.v = v;
-  rec.mat_ptr = mp.get();
+  rec.u = uHit;
+  rec.v = vHit;
+  
+  rec.mat_ptr = mesh->mtl_materials[mat_id].get();
   rec.alpha_miss = alpha_miss;
   rec.shape = this;
   
@@ -179,10 +202,15 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, rand
 }
 
 
-bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler *sampler) {
-  point3f p0t = a - vec3f(r.origin());
-  point3f p1t = b - vec3f(r.origin());
-  point3f p2t = c - vec3f(r.origin());
+bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler* sampler) {
+  const point3f &p0 = mesh->p[v[0]];
+  const point3f &p1 = mesh->p[v[1]];
+  const point3f &p2 = mesh->p[v[2]];
+  
+  point3f p0t = p0 - vec3f(r.origin());
+  point3f p1t = p1 - vec3f(r.origin());
+  point3f p2t = p2 - vec3f(r.origin());
+  
   int kz = MaxDimension(Abs(r.direction()));
   int kx = kz + 1;
   if (kx == 3) kx = 0;
@@ -264,19 +292,11 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Samp
   
   vec3f dpdu, dpdv;
   point2f uv[3];
-  if(uv_provided) {
-    uv[0] = uv_a;
-    uv[1] = uv_b;
-    uv[2] = uv_c;
-  } else {
-    uv[0] = point2f(0, 0);
-    uv[1] = point2f(1, 0);
-    uv[2] = point2f(1, 1);
-  }
+  GetUVs(uv);
   
   // Compute deltas for triangle partial derivatives
   vec2f duv02 = uv[0] - uv[2], duv12 = uv[1] - uv[2];
-  vec3f dp02 = a - c, dp12 = b - c;
+  vec3f dp02 = p0 - p2, dp12 = p1 - p2;
   Float determinant = DifferenceOfProducts(duv02[0],duv12[1],duv02[1],duv12[0]);
   bool degenerateUV = std::fabs(determinant) < 1e-8;
   if (!degenerateUV) {
@@ -286,7 +306,7 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Samp
   }
   if (degenerateUV || cross(rec.dpdu, rec.dpdv).squared_length() == 0) {
     // Handle zero determinant for triangle partial derivative matrix
-    vec3f ng = cross(c - a, b - a);
+    vec3f ng = cross(p2 - p0, p1 - p0);
     if (ng.squared_length() == 0) {
       // The triangle is actually degenerate; the intersection is
       // bogus.
@@ -295,24 +315,31 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Samp
     CoordinateSystem(unit_vector(ng), &rec.dpdu, &rec.dpdv);
   }
   //Add error calc
-  Float xAbsSum = (std::abs(b0 * a.x()) + std::abs(b1 * b.x()) +
-    std::abs(b2 * c.x()));
-  Float yAbsSum = (std::abs(b0 * a.y()) + std::abs(b1 * b.y()) +
-    std::abs(b2 * c.y()));
-  Float zAbsSum = (std::abs(b0 * a.z()) + std::abs(b1 * b.z()) +
-    std::abs(b2 * c.z()));
+  Float xAbsSum = (std::abs(b0 * p0.x()) + std::abs(b1 * p1.x()) +
+    std::abs(b2 * p2.x()));
+  Float yAbsSum = (std::abs(b0 * p0.y()) + std::abs(b1 * p1.y()) +
+    std::abs(b2 * p2.y()));
+  Float zAbsSum = (std::abs(b0 * p0.z()) + std::abs(b1 * p1.z()) +
+    std::abs(b2 * p2.z()));
   rec.pError = gamma(7) * vec3f(xAbsSum, yAbsSum, zAbsSum);
   
-  point3f pHit = b0 * a + b1 * b + b2 * c;
+  point3f pHit = b0 * p0 + b1 * p1 + b2 * p2;
   point2f uvHit = b0 * uv[0] + b1 * uv[1] + b2 * uv[2];
-
-  Float u = uvHit[0];
-  Float v = uvHit[1];
+  
+  Float uHit = uvHit[0];
+  Float vHit = uvHit[1];
+  if(mesh->has_vertex_colors) {
+    uHit = b0;
+    vHit = b1;
+  } 
   
   bool alpha_miss = false;
+  normal3f normal = normal3f(unit_vector(cross(dp02, dp12)));
+  int mat_id = mesh->face_material_id[face_number];
   
+  alpha_texture* alpha_mask = mesh->alpha_textures[mat_id].get();
   if(alpha_mask) {
-    if(alpha_mask->value(u, v, rec.p) < sampler->Get1D()) {
+    if(alpha_mask->value(uHit, vHit, rec.p) < sampler->Get1D()) {
       alpha_miss = true;
     }
   }
@@ -323,13 +350,29 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Samp
   rec.has_bump = false;
   
   // Use that to calculate normals
-  if(normals_provided) {
-    point2f uvHit2 = b0 *  point2f(0, 0) + b1 * point2f(1, 0) + b2 * point2f(1, 1);
-    Float u1 = uvHit2[0];
-    Float v1 = uvHit2[1];
-    Float w = 1 - u1 - v1;
-
-    rec.normal = unit_vector(w * na + u1 * nb + v1 * nc);
+  if(mesh->has_normals) {
+    normal3f n1 = mesh->n[n[0]];
+    normal3f n2 = mesh->n[n[1]];
+    normal3f n3 = mesh->n[n[2]];
+    normal3f np = unit_vector(b0 * n1 + b1 * n2 + b2 * n3);
+    
+    if(mesh->has_consistent_normals) {
+      bool flip = dot(np, r.direction()) > 0;
+      Float af1 = mesh->alpha_v[n[0]];
+      Float af2 = mesh->alpha_v[n[1]];
+      Float af3 = mesh->alpha_v[n[2]];
+      Float af = b0 * af1 + b1 * af2 + b2 * af3;
+      vec3f i = -unit_vector(r.direction());
+      i *= flip ? -1 : 1;
+      Float b = dot(i,np);
+      Float q = (1 - 2 * M_1_PI * af) * (1 - (2 * M_1_PI) * af)/(1 + 2 * (1 - 2 * M_1_PI) * af);
+      Float g = 1 + q * (b - 1);
+      Float rho = sqrt(q * (1 + g) / (1 +b));
+      normal3f r1 = (g + rho * b) * np - rho * normal3f(i.x(),i.y(),i.z());
+      rec.normal = unit_vector(normal3f(i.x(),i.y(),i.z()) + r1);
+    } else {
+      rec.normal = np;
+    }
   } else {
     if(alpha_mask) {
       rec.normal = dot(r.direction(), normal) < 0 ? normal : -normal;
@@ -337,19 +380,22 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Samp
       rec.normal = normal;
     }
   }
+  bump_texture* bump_tex = mesh->bump_textures[mat_id].get();
   
   if(bump_tex) {
-    vec3f norm_bump = dot(r.direction(), normal) < 0 ? rec.normal.convert_to_vec3() : -rec.normal.convert_to_vec3();
+    vec3f norm_bump = dot(r.direction(), rec.normal) < 0 ? rec.normal.convert_to_vec3() : -rec.normal.convert_to_vec3();
     
-    point3f bvbu = bump_tex->value(u, v, rec.p);
+    point3f bvbu = bump_tex->value(uHit, vHit, rec.p);
     rec.bump_normal = cross(rec.dpdu + bvbu.x() * norm_bump ,
-                            rec.dpdv - bvbu.y() * norm_bump );
+                            rec.dpdv - bvbu.y() * norm_bump);
     rec.bump_normal.make_unit_vector();
+    rec.bump_normal = Faceforward(rec.bump_normal,rec.normal);
     rec.has_bump = true;
   }
-  rec.u = u;
-  rec.v = v;
-  rec.mat_ptr = mp.get();
+  rec.u = uHit;
+  rec.v = vHit;
+  
+  rec.mat_ptr = mesh->mtl_materials[mat_id].get();
   rec.alpha_miss = alpha_miss;
   rec.shape = this;
   
@@ -358,19 +404,22 @@ bool triangle::hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Samp
 
 
 bool triangle::bounding_box(Float t0, Float t1, aabb& box) const {
-  point3f min_v(fmin(fmin(a.x(), b.x()), c.x()), 
-                fmin(fmin(a.y(), b.y()), c.y()), 
+  const point3f &a = mesh->p[v[0]];
+  const point3f &b = mesh->p[v[1]];
+  const point3f &c = mesh->p[v[2]];
+  point3f min_v(fmin(fmin(a.x(), b.x()), c.x()),
+                fmin(fmin(a.y(), b.y()), c.y()),
                 fmin(fmin(a.z(), b.z()), c.z()));
-  point3f max_v(fmax(fmax(a.x(), b.x()), c.x()), 
-                fmax(fmax(a.y(), b.y()), c.y()), 
+  point3f max_v(fmax(fmax(a.x(), b.x()), c.x()),
+                fmax(fmax(a.y(), b.y()), c.y()),
                 fmax(fmax(a.z(), b.z()), c.z()));
-  
+
   point3f difference = max_v + -min_v;
-  
+
   if (difference.x() < 1E-5) max_v.e[0] += 1E-5;
   if (difference.y() < 1E-5) max_v.e[1] += 1E-5;
   if (difference.z() < 1E-5) max_v.e[2] += 1E-5;
-  
+
   box = aabb(min_v, max_v);
   return(true);
 }
@@ -378,9 +427,7 @@ bool triangle::bounding_box(Float t0, Float t1, aabb& box) const {
 Float triangle::pdf_value(const point3f& o, const vec3f& v, random_gen& rng, Float time) { 
   hit_record rec;
   if (this->hit(ray(o, v), 0.001, FLT_MAX, rec, rng)) {
-    Float distance = rec.t * rec.t * v.squared_length();;
-    Float cosine = dot(v, rec.normal);
-    return(distance / (cosine * area));
+    return(1 / SolidAngle(o));
   }
   return 0; 
 }
@@ -388,26 +435,60 @@ Float triangle::pdf_value(const point3f& o, const vec3f& v, random_gen& rng, Flo
 Float triangle::pdf_value(const point3f& o, const vec3f& v, Sampler* sampler, Float time) { 
   hit_record rec;
   if (this->hit(ray(o, v), 0.001, FLT_MAX, rec, sampler)) {
-    Float distance = rec.t * rec.t * v.squared_length();;
-    Float cosine = dot(v, rec.normal);
-    return(distance / (cosine * area));
+    return(1 / SolidAngle(o));
   }
   return 0; 
 }
 
-vec3f triangle::random(const point3f& origin, random_gen& rng, Float time) {
-  Float r1 = rng.unif_rand();
-  Float r2 = rng.unif_rand();
-  Float sr1 = sqrt(r1);
-  point3f random_point((1.0 - sr1) * a + sr1 * (1.0 - r2) * b + sr1 * r2 * c);
-  return(random_point - origin); 
-}
-vec3f triangle::random(const point3f& origin, Sampler* sampler, Float time) {
-  vec2f u = sampler->Get2D();
-  Float r1 = u.x();
-  Float r2 = u.y();
-  Float sr1 = sqrt(r1);
-  point3f random_point((1.0 - sr1) * a + sr1 * (1.0 - r2) * b + sr1 * r2 * c);
-  return(random_point - origin); 
+Float triangle::SolidAngle(point3f p) const {
+  const point3f &a = mesh->p[v[0]];
+  const point3f &b = mesh->p[v[1]];
+  const point3f &c = mesh->p[v[2]];
+  return SphericalTriangleArea(unit_vector(a - p), 
+                               unit_vector(b - p),
+                               unit_vector(c - p));
 }
 
+vec3f triangle::random(const point3f& origin, random_gen& rng, Float time) {
+  const point3f &a = mesh->p[v[0]];
+  const point3f &b = mesh->p[v[1]];
+  const point3f &c = mesh->p[v[2]];
+  Float r1 = sqrt(rng.unif_rand());
+  Float r2 = rng.unif_rand();
+  Float u1 = 1.0f-r1;
+  Float u2 = r2*r1;
+  point3f random_point = (u1 * a + u2 * b + (1 - u1 - u2) * c);
+  return(random_point - origin);
+}
+vec3f triangle::random(const point3f& origin, Sampler* sampler, Float time) {
+  const point3f &a = mesh->p[v[0]];
+  const point3f &b = mesh->p[v[1]];
+  const point3f &c = mesh->p[v[2]];
+  vec2f u = sampler->Get2D();
+  Float r1 = sqrt(u.x());
+  Float r2 = u.y();
+  Float u1 = 1.0f-r1;
+  Float u2 = r2*r1;
+  point3f random_point = (u1 * a + u2 * b + (1 - u1 - u2) * c);
+  return(random_point - origin);
+}
+
+void triangle::GetUVs(point2f uv[3]) const {
+  if (mesh->has_tex && t[0] != -1 && t[1] != -1 && t[1] != -1) {
+    uv[0] = mesh->uv[t[0]];
+    uv[1] = mesh->uv[t[1]];
+    uv[2] = mesh->uv[t[2]];
+  } else {
+    uv[0] = point2f(0, 0);
+    uv[1] = point2f(1, 0);
+    uv[2] = point2f(1, 1);
+  }
+}
+
+Float triangle::Area() const {
+  // Get triangle vertices in _p0_, _p1_, and _p2_
+  const point3f &p0 = mesh->p[v[0]];
+  const point3f &p1 = mesh->p[v[1]];
+  const point3f &p2 = mesh->p[v[2]];
+  return 0.5 * cross(p1 - p0, p2 - p0).length();
+}
